import PropTypes from 'prop-types';
import React, { ChangeEvent, FocusEvent, KeyboardEvent, MouseEvent, SyntheticEvent } from 'react';
import { noop } from '../utils';
import type { Option, RefElement, SelectEvent, TypeaheadProps, TypeaheadState } from '../types';
interface Props extends Omit<TypeaheadProps, 'onChange' | 'selected'> {
    onChange?: (selected: Option[]) => void;
    selected?: Option[];
}
export declare function getInitialState(props: Props): TypeaheadState;
export declare function clearTypeahead(state: TypeaheadState, props: Props): {
    isFocused: boolean;
    selected: never[];
    text: string;
    activeIndex: number;
    activeItem?: Option | undefined;
    initialItem?: Option | undefined;
    showMenu: boolean;
    shownResults: number;
};
export declare function clickOrFocusInput(state: TypeaheadState): {
    isFocused: boolean;
    showMenu: boolean;
    activeIndex: number;
    activeItem?: Option | undefined;
    initialItem?: Option | undefined;
    selected: Option[];
    shownResults: number;
    text: string;
};
export declare function hideMenu(state: TypeaheadState, props: Props): {
    activeIndex: number;
    activeItem: Option | undefined;
    initialItem: Option | undefined;
    showMenu: boolean;
    shownResults: number;
    isFocused: boolean; /**
     * Whether the filter should ignore accents and other diacritical marks.
     */
    selected: Option[];
    text: string;
};
export declare function toggleMenu(state: TypeaheadState, props: Props): {
    showMenu: boolean;
    activeIndex: number;
    activeItem?: Option | undefined;
    initialItem?: Option | undefined;
    isFocused: boolean; /**
     * Whether the filter should ignore accents and other diacritical marks.
     */
    selected: Option[];
    shownResults: number;
    text: string;
};
declare class Typeahead extends React.Component<Props, TypeaheadState> {
    static propTypes: {
        /**
         * Allows the creation of new selections on the fly. Note that any new items
         * will be added to the list of selections, but not the list of original
         * options unless handled as such by `Typeahead`'s parent.
         *
         * If a function is specified, it will be used to determine whether a custom
         * option should be included. The return value should be true or false.
         */
        allowNew: PropTypes.Requireable<boolean | ((...args: any[]) => any)>;
        /**
         * Autofocus the input when the component initially mounts.
         */
        autoFocus: PropTypes.Requireable<boolean>;
        /**
         * Whether or not filtering should be case-sensitive.
         */
        caseSensitive: (props: TypeaheadProps, propName: keyof TypeaheadProps, componentName: string) => void;
        /**
         * The initial value displayed in the text input.
         */
        defaultInputValue: (props: TypeaheadProps, propName: keyof TypeaheadProps, componentName: string) => void;
        /**
         * Whether or not the menu is displayed upon initial render.
         */
        defaultOpen: PropTypes.Requireable<boolean>;
        /**
         * Specify any pre-selected options. Use only if you want the component to
         * be uncontrolled.
         */
        defaultSelected: (props: TypeaheadProps, propName: keyof TypeaheadProps, componentName: string) => void;
        /**
         * Either an array of fields in `option` to search, or a custom filtering
         * callback.
         */
        filterBy: PropTypes.Requireable<string[] | ((...args: any[]) => any)>;
        /**
         * Highlights the menu item if there is only one result and allows selecting
         * that item by hitting enter. Does not work with `allowNew`.
         */
        highlightOnlyResult: (props: TypeaheadProps, propName: keyof TypeaheadProps, componentName: string) => void;
        /**
         * An html id attribute, required for assistive technologies such as screen
         * readers.
         */
        id: (props: TypeaheadProps, propName: keyof TypeaheadProps, componentName: string) => void;
        /**
         * Whether the filter should ignore accents and other diacritical marks.
         */
        ignoreDiacritics: (props: TypeaheadProps, propName: keyof TypeaheadProps, componentName: string) => void;
        /**
         * Specify the option key to use for display or a function returning the
         * display string. By default, the selector will use the `label` key.
         */
        labelKey: (props: TypeaheadProps, propName: keyof TypeaheadProps, componentName: string) => void;
        /**
         * Maximum number of results to display by default. Mostly done for
         * performance reasons so as not to render too many DOM nodes in the case of
         * large data sets.
         */
        maxResults: PropTypes.Requireable<number>;
        /**
         * Number of input characters that must be entered before showing results.
         */
        minLength: PropTypes.Requireable<number>;
        /**
         * Whether or not multiple selections are allowed.
         */
        multiple: PropTypes.Requireable<boolean>;
        /**
         * Invoked when the input is blurred. Receives an event.
         */
        onBlur: PropTypes.Requireable<(...args: any[]) => any>;
        /**
         * Invoked whenever items are added or removed. Receives an array of the
         * selected options.
         */
        onChange: PropTypes.Requireable<(...args: any[]) => any>;
        /**
         * Invoked when the input is focused. Receives an event.
         */
        onFocus: PropTypes.Requireable<(...args: any[]) => any>;
        /**
         * Invoked when the input value changes. Receives the string value of the
         * input.
         */
        onInputChange: PropTypes.Requireable<(...args: any[]) => any>;
        /**
         * Invoked when a key is pressed. Receives an event.
         */
        onKeyDown: PropTypes.Requireable<(...args: any[]) => any>;
        /**
         * Invoked when menu visibility changes.
         */
        onMenuToggle: PropTypes.Requireable<(...args: any[]) => any>;
        /**
         * Invoked when the pagination menu item is clicked. Receives an event.
         */
        onPaginate: PropTypes.Requireable<(...args: any[]) => any>;
        /**
         * Whether or not the menu should be displayed. `undefined` allows the
         * component to control visibility, while `true` and `false` show and hide
         * the menu, respectively.
         */
        open: PropTypes.Requireable<boolean>;
        /**
         * Full set of options, including pre-selected options. Must either be an
         * array of objects (recommended) or strings.
         */
        options: PropTypes.Validator<(string | object | null | undefined)[]>;
        /**
         * Give user the ability to display additional results if the number of
         * results exceeds `maxResults`.
         */
        paginate: PropTypes.Requireable<boolean>;
        /**
         * The selected option(s) displayed in the input. Use this prop if you want
         * to control the component via its parent.
         */
        selected: (props: TypeaheadProps, propName: keyof TypeaheadProps, componentName: string) => void;
    };
    static defaultProps: {
        allowNew: boolean;
        autoFocus: boolean;
        caseSensitive: boolean;
        defaultInputValue: string;
        defaultOpen: boolean;
        defaultSelected: never[];
        filterBy: never[];
        highlightOnlyResult: boolean;
        ignoreDiacritics: boolean;
        labelKey: string;
        maxResults: number;
        minLength: number;
        multiple: boolean;
        onBlur: typeof noop;
        onFocus: typeof noop;
        onInputChange: typeof noop;
        onKeyDown: typeof noop;
        onMenuToggle: typeof noop;
        onPaginate: typeof noop;
        paginate: boolean;
    };
    state: TypeaheadState;
    inputNode: RefElement<HTMLInputElement>;
    isMenuShown: boolean;
    items: Option[];
    componentDidMount(): void;
    componentDidUpdate(prevProps: Props, prevState: TypeaheadState): void;
    render(): JSX.Element;
    blur: () => void;
    clear: () => void;
    focus: () => void;
    getInput: () => RefElement<HTMLInputElement>;
    inputRef: (inputNode: RefElement<HTMLInputElement>) => void;
    setItem: (item: Option, position: number) => void;
    hideMenu: () => void;
    toggleMenu: () => void;
    _handleActiveIndexChange: (activeIndex: number) => void;
    _handleActiveItemChange: (activeItem: Option) => void;
    _handleBlur: (e: FocusEvent<HTMLInputElement>) => void;
    _handleChange: (selected: Option[]) => void;
    _handleClear: () => void;
    _handleClick: (e: MouseEvent<HTMLInputElement>) => void;
    _handleFocus: (e: SyntheticEvent<HTMLInputElement>) => void;
    _handleInitialItemChange: (initialItem?: Option | undefined) => void;
    _handleInputChange: (e: ChangeEvent<HTMLInputElement>) => void;
    _handleKeyDown: (e: KeyboardEvent<HTMLInputElement>) => void;
    _handleMenuItemSelect: (option: Option, e: SelectEvent<HTMLElement>) => void;
    _handlePaginate: (e: SelectEvent<HTMLElement>) => void;
    _handleSelectionAdd: (option: Option) => void;
    _handleSelectionRemove: (selection: Option) => void;
}
export default Typeahead;
